package com.chilunyc.fumao.common.util.jpa.query;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.extern.log4j.Log4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.util.ReflectionUtils;

import javax.persistence.criteria.*;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 *  on 2018/7/21.
 */
@Log4j
public class QueryBuilder {

    @Data
    @AllArgsConstructor
    private static class QueryDesc {
        Field field;
        QueryItem queryItem;
    }

    /**
     * 查询缓存
     */
    private static Map<Class, List<QueryDesc>> classQueryDescMap = new ConcurrentHashMap<>();


    /**
     * @param object
     * @param
     * @return
     */
    public static Specification build(Object object) {


        /**
         *
         */
        return (root, criteriaQuery, cb) -> {
            Class clazz = object.getClass();
            List<QueryDesc> descMap = getQueryDescMap(clazz);
            List<Predicate> predicates = new ArrayList<>(descMap.size());
            descMap.forEach(desc -> {
                Object value = getValue(desc.field, object);

                // 为null不加入查询
                if (value == null && !desc.queryItem.nullable()) {
                    return;
                }

                // 字符串为""不加入查询
                if (value != null && value instanceof String && !desc.queryItem.blankable() && StringUtils.isBlank(value.toString())) {
                    return;
                }

                //
                String path = desc.queryItem.column();
                if (StringUtils.isBlank(path)) {
                    path = desc.field.getName();
                }

                // 有join查询
                if (!desc.queryItem.join().equals(QueryJoinType.None)) {
                    String[] split = StringUtils.split(path, ".");
                    Join join = null;
                    switch (desc.queryItem.join()) {
                        case Inner:
                            join = root.join(split[0], JoinType.INNER);
                            break;
                        case Left:
                            join = root.join(split[0], JoinType.LEFT);
                            break;
                        case Right:
                            join = root.join(split[0], JoinType.RIGHT);
                            break;
                    }
                    if (join != null) {
                        Predicate predicate = toCondition(desc.queryItem, value, cb, join.get(split[1]));
                        if (predicate != null) {
                            join.on(predicate);
                        }
                    }
                }

                // 没有join, 直接加条件
                else {
                    Predicate predicate = toCondition(desc.queryItem, value, cb, root.get(path));
                    if (predicate != null) {
                        predicates.add(predicate);
                    }
                }
            });

            SortItems sortItems = (SortItems) clazz.getAnnotation(SortItems.class);
            if (sortItems != null && sortItems.value() != null) {
                for (SortItem sortItem : sortItems.value()) {
                    if (sortItem.desc()) {
                        criteriaQuery.orderBy(cb.desc(root.get(sortItem.column())));
                    } else {
                        criteriaQuery.orderBy(cb.asc(root.get(sortItem.column())));
                    }
                }
            }
            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };
    }

    /**
     * 转成条件
     *
     * @param queryItem
     * @param value
     * @param cb
     * @param expression
     * @return
     */
    private static Predicate toCondition(QueryItem queryItem, Object value, CriteriaBuilder cb, Expression expression) {
        switch (queryItem.type()) {
            case in:
                CriteriaBuilder.In in = cb.in(expression);
                List list = (List) value;
                for (Object o : list) {
                    in.value(o);
                }
                return in;
            case equal:
                return cb.equal(expression, value);
            case like:
                return cb.like(expression, "%" + value + "%");
            case gt:
                return cb.gt(expression, (Number) value);
            case lt:
                return cb.lt(expression, (Number) value);
            case ge:
                return cb.ge(expression, (Number) value);
            case le:
                return cb.le(expression, (Number) value);
            case notEqual:
                return cb.notEqual(expression, value);
            case notLike:
                return cb.notLike(expression, "%" + value + "%");
            case rightLike:
                return cb.like(expression, value + "%");
            case betweenDate:
                Date[] timeRange = (Date[]) value;
                if (timeRange.length == 2) {
                    return cb.between(expression, timeRange[0], timeRange[1]);
                } else {
                    log.warn("betweenDate 需要两个日期");
                }
            case notNull:
                if (Boolean.valueOf(value.toString())) {
                    return cb.isNotNull(expression);
                }
        }
        return null;
    }

    /**
     * @param field
     * @param src
     * @return
     */
    private static Object getValue(Field field, Object src) {
        try {
            return field.get(src);
        } catch (IllegalAccessException e) {
            return null;
        }
    }

    /**
     * @param clazz
     * @return
     */
    private static List<QueryDesc> getQueryDescMap(Class clazz) {
        List<QueryDesc> queryDescs = classQueryDescMap.get(clazz);
        if (queryDescs != null) {
            return queryDescs;
        }
        synchronized (clazz.getName()) {
            queryDescs = classQueryDescMap.get(clazz);
            if (queryDescs != null) {
                return queryDescs;
            }

            // 组装
            List<QueryDesc> real = new ArrayList<>();
            ReflectionUtils.doWithFields(clazz, (field) -> {
                QueryDesc queryDec = getQueryDec(field);
                if (queryDec == null) {
                    return;
                }
                field.setAccessible(true);
                real.add(queryDec);
            });

            //
            return real;
        }
    }

    /**
     * @param field
     * @return
     */
    private static QueryDesc getQueryDec(Field field) {
        QueryItem annotation = field.getAnnotation(QueryItem.class);
        if (annotation == null) {
            return null;
        }
        return new QueryDesc(field, annotation);
    }
}
